#include <sys/types.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <assert.h>

#include "transcoder.h"

typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned int u32;

#define MAX_COUNT 2048

int main(int argc, char **argv)
{
	int task_id;
	int fd = -1;
	int i = 0, n = 0, ret = 0;
	struct mem_info ep_region[MAX_COUNT];
	int mem_size = 5;

	if (argc < 2) {
		printf("DDR malloc and free test\n");
		printf("command line: %s <node> <size in MB(%d)>.\n", argv[0], mem_size);
		printf("example: %s /dev/ama_transcoder0 100\n", argv[0]);
		return -1;
	}

	fd = open(argv[1], O_RDWR);
	if (fd == -1) {
		printf("Failed to open dev: %s\n", argv[1]);
		goto end;
	} else
		printf("open /dev/trans_mem succeed.\n");

	if (argc < 3) {
		printf("use default mem_size=%dMB\n", mem_size);
	} else
		mem_size = strtoul(argv[2], 0, 0);

	ioctl(fd, SN_TRANX_MEM_GET_TASKID, &task_id);
	printf("get task id = %d\n", task_id);

	while( !ret && n < MAX_COUNT) {
        ep_region[n].size = 1024 * 1024 * mem_size;
        ep_region[n].task_id = task_id;
		ret = ioctl(fd, SN_TRANX_MEM_ALLOC,
			    &ep_region[n]);
		if (ret) {
			printf("SN_TRANX_MEM_ALLOC failed（%d).\n", ret);
			goto end;
		}
		printf("EP: alloc %d phy_addr=0x%llx size=0x%x, total=%dMB\n", n,
		       ep_region[n].phy_addr, ep_region[n].size, mem_size * n);
        n++;
		usleep(10000);
	}

end:
	printf("Malloc done\n");
	for (i = 0; i <= n; i++) {
		ioctl(fd, SN_TRANX_MEM_FREE, &ep_region[i]);
		printf("EP: free %d phy_addr=0x%llx size=0x%x.\n", i,
		       ep_region[i].phy_addr, ep_region[i].size);
	}
	printf("Free done\n");

	ioctl(fd, SN_TRANX_MEM_FREE_TASKID, &task_id);

	close(fd);
	return 0;
}
